@@ -4,6 +4,9 @@ class BeakerException(Exception):
     pass
 
 
+class BeakerWarning(RuntimeWarning):
+    """Issued at runtime."""
+
 class CreationAbortedError(Exception):
     """Deprecated."""
 
@@ -20,8 +23,5 @@ class LockError(BeakerException):
     pass
 
 
-class InvalidCryptoBackendError(BeakerException, ImportError):
-
-    def __init__(self):
-        Exception.__init__(self,
-                           'No supported crypto implementation was found')
+class InvalidCryptoBackendError(BeakerException):
+    pass
